"""
Tests for casa_envoy Auditor
"""
from pathlib import Path
from unittest.mock import mock_open, patch

from casa_envoy.auditor import (
    AuditDirectories,
    AuditFiles,
    get_fields_for,
    get_value_for,
    get_xml_content,
)
from casa_envoy.schema import AbstractTextFile

cal_settings = {
    "useCasa": False,
    "homeForReprocessing": "/home/casa/packages/pipeline/current",
    "rootDirectory": "/tmp/workspaces_tmp/",
    "processingDirectory": "tmpiox5trbp",
    "metadata": "test/input_files/test.json",
    "ppr": "test/input_files/PPR.xml",
    "product_type": "standard-cal",
}
test_ppr = AbstractTextFile(
    filename="test/input_files/PPR.xml", content=Path("test/input_files/PPR.xml").read_text()
)
test_cal_metadata = AbstractTextFile(
    filename="test/input_files/test.json", content=Path("test/input_files/test.json").read_text()
)

img_settings = {
    "useCasa": False,
    "homeForReprocessing": "/home/casa/packages/pipeline/current",
    "rootDirectory": "/tmp/workspaces_tmp/",
    "processingDirectory": "tmpiox5trbp",
    "metadata": "test/input_files/image-metadata.json",
    "ppr": "test/input_files/cmsimage-PPR.xml",
    "product_type": "standard-img",
}
test_img_ppr = AbstractTextFile(
    filename="test/input_files/cmsimage-PPR.xml",
    content=Path("test/input_files/cmsimage-PPR.xml").read_text(),
)
test_img_metadata = AbstractTextFile(
    filename="test/input_files/image-metadata.json",
    content=Path("test/input_files/image-metadata.json").read_text(),
)


def test_get_fields_for():
    product_type = "standard-cal"
    fields = [
        "fileSetIds",
        "workflowName",
        "systemId",
        "creationTime",
        "productLocator",
        "destinationDirectory",
    ]
    result = get_fields_for(product_type=product_type, filename=test_cal_metadata.filename)
    assert result == fields
    fields2 = ["RootDirectory", "RelativePath", "SdmIdentifier"]
    result2 = get_fields_for(product_type=product_type, filename=test_ppr.filename)
    assert result2 == fields2

    product_type2 = "standard-img"
    img_fields = [
        "fileSetIds",
        "workflowName",
        "systemId",
        "creationTime",
        "productLocator",
        "destinationDirectory",
        "calibrationSourceDirectory",
        "cmsName",
    ]
    result = get_fields_for(product_type=product_type2, filename=test_img_metadata.filename)
    assert result == img_fields


def test_get_xml_content():
    content = get_xml_content(test_ppr)
    assert content.find("RelativePath").string == "tmpiox5trbp"
    assert content.find("SdmIdentifier").string == "brain_001.58099.678886747686"
    assert content.find("CreationTime").string == "2021-05-06T17:34:36"


def test_get_value_for():
    value = get_value_for(test_ppr, "RelativePath")
    assert value == "tmpiox5trbp"
    value = get_value_for(test_ppr, "SdmIdentifier")
    assert value == "brain_001.58099.678886747686"

    value = get_value_for(test_cal_metadata, "workflowName")
    assert value == "std_calibration"
    value = get_value_for(test_cal_metadata, "productLocator")
    assert value == "uid://evla/execblock/9cb0964b-ad6b-40ed-bd87-d08c502503e2"


class TestAuditFiles:
    def test_check_required_fields(self):
        audit_files = AuditFiles([test_cal_metadata, test_ppr], cal_settings)
        fields = [
            "fileSetIds",
            "workflowName",
            "systemId",
            "creationTime",
            "productLocator",
            "destinationDirectory",
        ]

        result = audit_files.check_required_fields(test_cal_metadata, fields)
        assert result is True

        fieldsF = [
            "fileSetIds",
            "workflowName",
            "systemId",
            "creationTime",
            "productLocator",
            "destinationDirectory",
            "FakeField",
        ]

        resultF = audit_files.check_required_fields(test_cal_metadata, fieldsF)
        assert resultF is False

        fields2 = ["RootDirectory", "RelativePath", "SdmIdentifier"]

        result2 = audit_files.check_required_fields(test_ppr, fields2)
        assert result2 is True

        fields2F = ["RootDirectory", "RelativePath", "SdmIdentifier", "FakeField"]

        result2F = audit_files.check_required_fields(test_ppr, fields2F)
        assert result2F is False

    @patch("os.chdir")
    @patch("shutil.copy")
    def test_correct_for_condor(self, mock_copy, mock_os):
        ppr = AuditFiles([test_cal_metadata, test_ppr], cal_settings).correct_for_condor(
            ppr=test_ppr
        )
        assert ppr.filename == "test/input_files/PPR.xml"

    def test_audit(self):
        result = AuditFiles([test_cal_metadata, test_ppr], cal_settings).audit()
        assert result is True


class TestAuditDirectories:
    def test_audit(self):
        result = AuditDirectories(test_ppr, cal_settings).audit
        assert result is False
